/*
 * $Id: main.c,v 1.1 1998/06/15 04:17:25 elkner Exp $
 *
 * Author:  Jens Elkner            elkner@ivs.cs.uni-magdeburg.de
 * Project: Jesalfa     http://ivs.cs.uni-magdeburg.de/~elkner/webtools/jesalfa/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * http://www.gnu.org/copyleft/gpl.html or ./gpl.html
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "main.h"
#include "signals.h"
#include "debug.h"
#include "version.h"
#include "tree.h"
#include "ip_list.h"
#include "utils.h"
#include "fparse.h"
#include "register.h"
#include "results.h"

static void Usage (void);
static void GetOptions(int argc, char *argv[]);

int debuglevel = 0;                /* set via Option -d */
long memusage = 0;                 /* current usage of dynam. alloc. memory */
long maxmem = 0;                   /* Max. of memusage during the run */
int numeric = 0;                   /* output in % or exact */
int verbose = 0;		   /* verbose output */

struct timeval starttime = {0,0};  
struct timeval stoptime = {0,0};

char *outfile = NULL;
char *outpath = NULL;
FILE *errfd;
char *cfgfile = NULL;
char *ccfile = NULL;
char section[128];

int do_domain = 0;
int do_content = 0;
int do_extension = 0;
int do_proto = 0;

static void
Usage(void)
{
    
    fprintf(stderr,
	    "Usage: %s [-f file] [-o file] [-p path] [-d level] "
	    "[-h] [-a] [-v] [logfile]\n\n"
	    "   -f file    use file as configuration file instead of .jrc\n"
	    "   -o file    file for storing debug and error messages\n"
	    "   -p path    prefix each output file with path\n"
	    "   -d level   Debug Level (default = 0)\n"
	    "   -c file    country codes file (default=country-codes)\n"
	    "   -h         print this help text\n"
	    "   -a         show invalid entries found in access log file\n"
	    "   -v         show version\n",
	    APPNAME);
    exit(1);
}

static void
GetOptions(int argc, char *argv[])
{
    extern char *optarg;
    extern int optind;
    int c;
    
    while ((c = getopt(argc, argv, "f:o:p:d:c:hav")) != -1) {
	switch (c) {
	    case 'f' : {
		cfgfile = savestr(optarg);
		break;
	    }
	    case 'o' : {
		outfile = savestr(optarg);
		break;
	    }
	    case 'p' : {
		outpath = savestr(optarg);
		break;
	    }
	    case 'd' : {
		debuglevel = atoi(optarg);
		break;
	    }
	    case 'c' : {
		ccfile = savestr(optarg);
		break;
	    }
	    case 'a' : {
		verbose++;
		break;
	    }
	    case 'v' : {
		printf("%s Version %s  (C) by %s  (%s)\n",
		       APPNAME, VERSION, AUTHOR, AUTHOR_EMAIL);
		exit(0);
	    }
	    default:
		Usage();
		break;
	}
    }
}

int
main(int argc, char **argv)
{
    int errcount = 0;
    char *filename;
    char buffer[1024];
    extern int optind;
    ip_acl *ip_list = NULL;
    
    strcpy(section,"main");
    SetStartTime();
    signal(SIGSEGV,SEGVhandler);
    signal(SIGBUS,SEGVhandler);
    GetOptions(argc, argv);
    if ( argv[optind] ) filename = savestr(argv[optind]);
    else filename = savestr("-\0");
    if (outfile) {
	if ( outpath ) {
	    strcpy(buffer,outpath);
	    strcat(buffer,"/");
	    strcat(buffer,outfile);
	}
	else
	    strcpy(buffer,outfile);
	if ( strcmp(filename,buffer) == 0) {
	    fprintf(stderr,"Warning: Access log file and error log file"
		    "seems to be the same: Using stderr!\n");
	    errfd = stderr;
	}
	else 
	    if ((errfd = fopen(buffer, "w")) == NULL) {
		fprintf(stderr,"Warning: Can't open file %s for writing! \n"
			"Using stderr!\n",buffer);
		errfd = stderr;
	    }
	delstr(outfile);
    }
    else
	errfd = stderr;
    if (cfgfile) {
	errcount = ReadrcFile(cfgfile, &ip_list);
	delstr(cfgfile);
    }
    else
	errcount = ReadrcFile(".jrc", &ip_list);
    if (errcount) {
	 CleanLogtypes(&logtype_root);
	 exit(1);
    }
    if (do_domain) {
	if (! ccfile )
	    ccfile = savestr("country-codes");
	ReadCountryCodes(ccfile);
	delstr(ccfile);
    }
    if (do_content)
	tree_init(&ctld_root);
    if (do_extension)
	tree_init(&xsld_root);
    if (do_proto)
	tree_init(&psld_root);
    ParseFile(filename, ip_list);
    delstr(filename);
    DeleteEmptyTLDs();
    PrintResults(ip_list);
    DeleteWordList(outfilelist);
    DeleteWordList(protolist);
    CleanTLDs(&tld_root);
    CleanTLDs(&ctld_root);
    CleanSLDs(&xsld_root);
    CleanSLDs(&psld_root);
    CleanLogtypes(&logtype_root);
    ip_acl_destroy(&ip_list);
    SetStopTime();
    GetTimeDiff(&starttime,&stoptime,buffer);
    fprintf(errfd,"%d lines analyzed in %s",
	    lineno,buffer);
    fprintf(errfd,"\nMax. dynamic memory usage (malloc): %ld Bytes ",
	    maxmem);
    fprintf(errfd,"\nfor %d TLDs and %d SLDs a %d groups.\n",
	    maxtld,maxsld,maxlogtypegrpid);
    fprintf(errfd,"Max. TLD length: %d, max. SLD length: %d\n",
	    maxtldlen,maxsldlen);
    close(errfd);
}
